import {Request, Response} from "express";
import {LogsUtils} from "@utils/logs.util";
import {HttpStatusCode} from "axios";
import {IRegisterInput} from "@interfaces/services/register.types";
import ValidatorsUtils from "@utils/validators.util";
import UsersService from "@services/users.service";
import RegisterService from "@services/authentication/register.service";

const logs = new LogsUtils('AuthController')

async function registerController(req: Request, res: Response) {
	const body = req.body;
	if (!body.username || !body.email || !body.password) {
		logs.warn('Missing required fields', req.ip);
		return res.status(HttpStatusCode.BadRequest).json({ error: 'Missing required fields' });
	}
	if (!ValidatorsUtils.isEmail(body.email)) {
		logs.warn('Invalid email format', req.ip);
		return res.status(HttpStatusCode.BadRequest).json({ error: 'Invalid email format' });
	}
	if (!ValidatorsUtils.isUsername(body.username)) {
		logs.warn('Invalid username format', req.ip);
		return res.status(HttpStatusCode.BadRequest).json({ error: 'Invalid username format' });
	}
	if (!ValidatorsUtils.isPassword(body.password)) {
		logs.warn('Invalid password format', req.ip);
		return res.status(HttpStatusCode.BadRequest).json({ error: 'Invalid password format' });
	}

	const UserIfExist = {
		byEmail: await UsersService.get.byEmail(body.email),
		byUsername: await UsersService.get.byUsername(body.username)
	}

	if (UserIfExist.byEmail.length > 0 || UserIfExist.byUsername.length > 0) {
		logs.warn('User already exists', req.ip);
    return res.status(HttpStatusCode.Found).json({ error: 'User already exists' });
	}

	const data: IRegisterInput = {
		username: body.username,
    email: body.email,
    password: body.password
	}
	if (body.displayName) data.displayName = body.displayName;

	const registerResult = await RegisterService(data)

	if (!registerResult.success) {
    logs.error(registerResult.message, req.ip);
		return res.status(HttpStatusCode.InternalServerError).json({ error: registerResult.message });
  }

	//TODO Image handling


	logs.info('User registered successfully', req.ip);
	return res.status(HttpStatusCode.Created).json({
		message: 'User registered successfully',
		token: registerResult.token,
		id: registerResult.id
	});
}


const AuthController = {
	register: registerController
}

export default AuthController;